<?php

namespace MotoPress\Appointment\Crons;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Such crons process all data in tasks (chunks) using a post-requests chain.
 * This approach allows you to avoid disabling the cron script on low
 * traffic sites because of the long execution time of free hosting.
 */
abstract class AbstractTaskChainCron extends AbstractWPCron {

	const TASKS_PROCESSING_TIME_LIMIT_IN_SECONDS = 20;


	public function __construct() {

		parent::__construct();

		add_action(
			'wp_ajax_' . static::getCronActionHookName(),
			function() {
				$this->processHttpPostRequestChain();
			}
		);
		add_action(
			'wp_ajax_nopriv_' . static::getCronActionHookName(),
			function() {
				$this->processHttpPostRequestChain();
			}
		);

		// make sure cron scheduled itself on the right time
		// and make sure we have cron as healthcheck process just in case
		// when task chain execution is terminated unexpectedly
		static::schedule();
	}


	public static function getCronStartIntervalId(): string {
		return static::CRON_START_INTERVAL_ID_EVERY_10_MIN;
	}


	final protected function executeCron() {

		// Start/restart the task chain execution if it is not already processing

		if ( static::isTaskChainExecutingNow() ) {
			exit;
		}

		$this->executeTasks( $this->getStartTaskData() );

		exit;
	}

	private function processHttpPostRequestChain() {

		try {

			// Don't lock up other requests while processing
			session_write_close();

			if ( static::isTaskChainExecutingNow() ) {
				wp_die();
			}

			check_ajax_referer( static::getCronActionHookName(), 'nonce' );

			$this->executeTasks( $_POST );

		} catch ( \Throwable $e ) {
			// phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_error_log
			error_log( $e->getMessage() . PHP_EOL . $e->getTraceAsString() );
		}

		wp_die();
	}


	public static function isTaskChainExecutingNow(): bool {

		return false !== get_site_transient( static::getCronActionHookName() . '_lock' );
	}


	private function sendHttpPostRequestToContinueTaskChain( array $nextTaskData = array() ) {

		// initate next chunk to be able to execute it without cron system
		$ajaxUrlForRequestsChaine = add_query_arg(
			array(
				'action' => static::getCronActionHookName(),
				'nonce'  => wp_create_nonce( static::getCronActionHookName() ),
			),
			admin_url( 'admin-ajax.php' )
		);

		return wp_remote_post(
			esc_url_raw( $ajaxUrlForRequestsChaine ),
			array(
				'timeout'   => 0.01,
				'blocking'  => false,
				'body'      => $nextTaskData,
				'cookies'   => $_COOKIE,
				'sslverify' => apply_filters( 'https_local_ssl_verify', false ),
			)
		);
	}

	/**
	 * Cron can send POST data to itself in http post request chain processing.
	 * For example we can find necessary data for tasks and send unprocessed part
	 * of it to the next cron execution to save time on the next step.
	 */
	abstract protected function getStartTaskData(): array;

	/**
	 * Allows to reschedule cron to the new correct time for optimisation
	 * after all tasks in the current task chain have been processed.
	 */
	protected function rescheduleCronToTheRightTime() {}

	/**
	 * Some small part (less than 25 seconds) of cron job must be executed here.
	 * If nothing left for processing - reschedule cron here to the right time for optimization!
	 * @return bool true - if anything left for processing otherwise - false
	 */
	protected function executeTasks( array $nextTaskData = array() ) {

		if ( empty( $nextTaskData ) ) {

			// nothing to process
			wp_die();
		}

		// Lock tasks execution of tasks to ensure that other threads
		// of this cron are not doing anything at the same time.
		set_site_transient(
			static::getCronActionHookName() . '_lock',
			microtime(),
			static::TASKS_PROCESSING_TIME_LIMIT_IN_SECONDS + 10 // seconds
		);

		// Set start time of current task processing step
		$finishTaskProcessingTime = time() + static::TASKS_PROCESSING_TIME_LIMIT_IN_SECONDS;

		do {
			// if task returns empty $nextTaskData then execution is over
			$nextTaskData = $this->processTask( $nextTaskData );

		} while ( $finishTaskProcessingTime > time() && ! empty( $nextTaskData ) );

		// Unlock tasks execution for other threads of this cron
		delete_site_transient( static::getCronActionHookName() . '_lock' );

		if ( ! empty( $nextTaskData ) ) {

			$this->sendHttpPostRequestToContinueTaskChain( $nextTaskData );

		} else {

			$this->rescheduleCronToTheRightTime();
		}

		wp_die();
	}

	/**
	 * Process single task of the current task chain. It has to as fast as possible!
	 * @return array of data for the next tasks in the chain that still need to be processed.
	 * If the returned array is empty, the task chain stops.
	 */
	abstract protected function processTask( array $taskData ): array;
}
