<?php

namespace MotoPress\Appointment\Entities;

use WP_Comment;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * @since 1.0
 *
 * @see \MotoPress\Appointment\Repositories\BookingRepository
 */
class Booking extends AbstractUniqueEntity {

	/**
	 * DO NOT USE THIS FIELD DIRECTLY
	 * IT WILL BE PROTECTED USE METHODS
	 * TODO: set it protected after woo payments plugin fix
	 * @var string
	 *
	 * @since 1.0
	 */
	public $status = 'new';

	/**
	 * @var float
	 *
	 * @since 1.0
	 */
	protected $totalPrice = 0.0;

	/**
	 * @var Reservation[]
	 *
	 * @since 1.0
	 */
	protected $reservations = array();

	/**
	 * DO NOT USE THIS FIELD DIRECTLY
	 * IT WILL BE PROTECTED USE METHODS
	 * TODO: set it protected after woo payments plugin fix
	 * @var Customer
	 *
	 * @since 1.0
	 */
	public $customer = null;

	/**
	 * @var string
	 *
	 * @since 1.10.2
	 */
	protected $customerNotes = '';

	/**
	 * @var int
	 *
	 * @since 1.11.0
	 */
	protected $couponId = 0;


	public function getStatus(): string {
		return $this->status;
	}

	public function setStatus( string $status ) {
		$this->status = $status;
	}

	/**
	 * @since 1.5.0
	 *
	 * @return float
	 */
	public function getTotalPrice() {
		return $this->totalPrice;
	}

	public function setTotalPrice( float $totalPrice ) {
		$this->totalPrice = $totalPrice;
	}

	/**
	 * @since 1.5.0
	 *
	 * @return float
	 */
	public function getPaidPrice() {
		$paidPrice = array_reduce(
			$this->getPayments(),
			function ( $carry, $payment ) {
				if ( $payment->isCompleted() ) {
					return $carry + $payment->getAmount();
				} else {
					return $carry;
				}
			},
			0.0
		);

		return $paidPrice;
	}

	/**
	 * @since 1.5.0
	 *
	 * @return float
	 */
	public function getToPayPrice() {
		return max( 0, $this->getTotalPrice() - $this->getPaidPrice() );
	}

	/**
	 * @since 1.11.0
	 *
	 * @return Reservation[]
	 */
	public function getReservations(): array {
		return $this->reservations;
	}

	public function setReservations( array $reservations ) {
		$this->reservations = $reservations;
	}

	/**
	 * @since 1.13.0
	 */
	public function getCustomer(): Customer {
		return $this->customer;
	}

	public function getCustomerNotes(): string {
		return $this->customerNotes;
	}

	/**
	 * @since 1.11.0
	 *
	 * @return Coupon|null
	 */
	public function getCoupon() {
		return mpapp()->repositories()->coupon()->findById( $this->couponId );
	}

	/**
	 * @since 1.11.0
	 *
	 * @return int
	 */
	public function getCouponId(): int {
		return $this->couponId;
	}

	/**
	 * @since 1.11.0
	 *
	 * @param int $couponId
	 */
	public function setCouponId( int $couponId ) {
		$this->couponId = $couponId;
	}

	/**
	 * @since 1.11.0
	 *
	 * @return bool
	 */
	public function hasCoupon(): bool {
		return 0 != $this->couponId;
	}

	/**
	 * @since 1.5.0
	 *
	 * @param Payment|int $payment
	 */
	public function expectPayment( $payment ) {
		$paymentId = is_object( $payment ) ? $payment->getId() : $payment;

		update_post_meta( $this->id, '_mpa_expect_payment', $paymentId );
	}

	/**
	 * @since 1.5.0
	 *
	 * @param Payment|int $payment
	 * @return bool
	 */
	public function isExpectsPayment( $payment ) {
		$paymentId = is_object( $payment ) ? $payment->getId() : $payment;

		return $paymentId === $this->getExpectingPaymentId();
	}

	/**
	 * @since 1.5.0
	 *
	 * @return int
	 */
	public function getExpectingPaymentId() {
		$expectPayment = get_post_meta( $this->id, '_mpa_expect_payment', true );

		if ( '' !== $expectPayment ) {
			return (int) $expectPayment;
		} else {
			return 0;
		}
	}

	/**
	 * @since 1.5.0
	 *
	 * @return Payment|null
	 */
	public function getExpectingPayment() {
		return mpapp()->repositories()->payment()->findById( $this->getExpectingPaymentId() );
	}

	/**
	 * @since 1.5.0
	 *
	 * @return Payment[]
	 */
	public function getPayments() {
		return mpapp()->repositories()->payment()->findAllByBooking( $this->id );
	}

	/**
	 * @param array $args Optional.
	 * @return array
	 *
	 * @see \MotoPress\Appointment\Repositories\AbstractRepository::findAll()
	 *
	 * @since 1.0
	 */
	public function getServices( $args = array() ) {
		return $this->getLinkedEntities( $this->getServiceIds(), $args, 'mpa_get_services' );
	}

	/**
	 * @param array $args Optional.
	 * @return array
	 *
	 * @see \MotoPress\Appointment\Repositories\AbstractRepository::findAll()
	 *
	 * @since 1.0
	 */
	public function getEmployees( $args = array() ) {
		return $this->getLinkedEntities( $this->getEmployeeIds(), $args, 'mpa_get_employees' );
	}

	/**
	 * @param array $args Optional.
	 * @return array
	 *
	 * @see \MotoPress\Appointment\Repositories\AbstractRepository::findAll()
	 *
	 * @since 1.0
	 */
	public function getLocations( $args = array() ) {
		return $this->getLinkedEntities( $this->getLocationIds(), $args, 'mpa_get_locations' );
	}

	/**
	 * @param int[] $ids
	 * @param array $args
	 * @param string $getCallback
	 * @return array
	 *
	 * @since 1.0
	 */
	protected function getLinkedEntities( $ids, $args, $getCallback ) {
		if ( empty( $ids ) ) {
			return array();
		}

		$searchArgs = array_merge( $args, array( 'include' => $ids ) );

		// mpa_get_services($searchArgs) etc.
		return $getCallback( $searchArgs );
	}

	/**
	 * @return int[]
	 *
	 * @since 1.0
	 */
	public function getServiceIds() {

		return array_map(
			function ( $reservation ) {
				return $reservation->getServiceId();
			},
			$this->reservations
		);
	}

	/**
	 * @return int[]
	 *
	 * @since 1.0
	 */
	public function getEmployeeIds() {

		return array_map(
			function ( $reservation ) {
				return $reservation->getEmployeeId();
			},
			$this->reservations
		);
	}

	/**
	 * @return int[]
	 *
	 * @since 1.0
	 */
	public function getLocationIds() {

		return array_map(
			function ( $reservation ) {
				return $reservation->getLocationId();
			},
			$this->reservations
		);
	}

	/**
	 * @return bool
	 *
	 * @since 1.10.1
	 */
	public function isAdminBooking(): bool {
		$isAdminBooking = get_post_meta( $this->id, '_mpa_is_admin_booking', true );

		return (bool) $isAdminBooking;
	}

	/**
	 * @return bool
	 *
	 * @since 1.11.0
	 */
	public function isConfirmed(): bool {
		return array_key_exists( $this->status, mpapp()->postTypes()->booking()->statuses()->getBookedStatuses() );
	}

	/**
	 * @param int|null $authorId Optional. Current logged in user ID by default.
	 * @return int|false The new comment's ID on success, false on failure.
	 *
	 * @since 1.14.0
	 */
	public function addLog( string $message, $authorId = null ) {
		return mpapp()->postTypes()->booking()->logs()->addLog( $this->id, $message, $authorId );
	}

	/**
	 * @return WP_Comment[] Logs in descending order.
	 *
	 * @since 1.14.0
	 */
	public function getLogs(): array {
		return mpapp()->postTypes()->booking()->logs()->getLogs( $this->id );
	}
}
