<?php

declare(strict_types=1);

namespace MotoPress\Appointment\Entities;

use MotoPress\Appointment\PostTypes\Statuses\AbstractPostStatuses;
use DateTime;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * @since 1.11.0
 */
class Coupon extends AbstractEntity {

	/**
	 * @var string
	 */
	protected $status = 'new';

	/**
	 * @var string
	 */
	protected $code = '';

	/**
	 * @var string
	 */
	protected $description = '';

	/**
	 * @var string
	 */
	protected $type = 'fixed';

	/**
	 * @var float
	 */
	protected $amount = 0.0;

	/**
	 * @var DateTime|null
	 */
	protected $expirationDate = null;

	/**
	 * @var int[]
	 */
	protected $serviceIds = array();

	/**
	 * @var DateTime|null
	 */
	protected $minDate = null;

	/**
	 * @var DateTime|null
	 */
	protected $maxDate = null;

	/**
	 * @var int
	 */
	protected $usageLimit = 0;

	/**
	 * @var int
	 */
	protected $usageCount = 0;

	public function getStatus(): string {
		return $this->status;
	}

	public function getCode(): string {
		return $this->code;
	}

	public function getDescription(): string {
		return $this->description;
	}

	public function getType(): string {
		return $this->type;
	}

	public function getAmount(): float {
		return $this->amount;
	}

	/**
	 * @return DateTime|null
	 */
	public function getExpirationDate() {
		return $this->expirationDate;
	}

	/**
	 * @return int[]
	 */
	public function getServiceIds(): array {
		return $this->serviceIds;
	}

	/**
	 * @return DateTime|null
	 */
	public function getMinDate() {
		return $this->minDate;
	}

	/**
	 * @return DateTime|null
	 */
	public function getMaxDate() {
		return $this->maxDate;
	}

	public function getUsageLimit(): int {
		return $this->usageLimit;
	}

	public function getUsageCount(): int {
		return $this->usageCount;
	}

	public function isValid(): bool {
		return $this->isPublic()
			&& ! $this->isExpired()
			&& ! $this->isExceededUsageLimit();
	}

	public function isPublic(): bool {
		return AbstractPostStatuses::STATUS_PUBLISH == $this->status;
	}

	public function isExpired(): bool {
		return ! is_null( $this->expirationDate )
			&& mpa_format_date( $this->expirationDate, 'internal' ) <= current_time( 'Y-m-d' );
	}

	public function isExceededUsageLimit(): bool {
		return $this->usageLimit > 0 && $this->usageCount >= $this->usageLimit;
	}

	public function isApplicableForBooking( Booking $booking ): bool {
		if ( ! $this->isValid() ) {
			return false;
		}

		foreach ( $booking->getReservations() as $reservation ) {
			if ( $this->isApplicableForReservation( $reservation ) ) {
				return true;
			}
		}

		return false;
	}

	public function isApplicableForReservation( Reservation $reservation ): bool {
		// Zero selected services = all services are available
		if ( ! empty( $this->serviceIds ) && ! in_array( $reservation->getServiceId(), $this->serviceIds ) ) {
			return false;
		}

		if ( ! is_null( $this->minDate ) && $reservation->getDate() < $this->minDate ) {
			return false;
		}

		if ( ! is_null( $this->maxDate ) && $reservation->getDate() > $this->maxDate ) {
			return false;
		}

		return true;
	}

	/**
	 * <b>Notice:</b> method does not take into account the value of
	 * Booking::$totalPrice, so the discount price may be higher.
	 */
	public function calcDiscountAmount( Booking $booking ): float {
		if ( ! $this->isValid() ) {
			return 0.0;
		} else {
			return $this->calcDiscountForBooking( $booking );
		}

		// Don't check the total price here:
		//     min($discountPrice, $booking->getTotalPrice())
		//
		// We also use this method to calculate the price of a new booking: you
		// cannot check the total price when calculating the total price. Booking
		// ID is also not a hint here...
	}

	protected function calcDiscountForBooking( Booking $booking ): float {
		$discount = 0.0;

		foreach ( $booking->getReservations() as $reservation ) {
			$discount += $this->calcDiscountForReservation( $reservation );
		}

		return $discount;
	}

	public function calcDiscountForReservation( Reservation $reservation ): float {
		$discountPrice = 0.0;

		if ( $this->isApplicableForReservation( $reservation ) ) {
			$itemPrice = $reservation->getPrice();

			switch ( $this->type ) {
				case 'fixed':
					$discountPrice = $this->amount;
					break;
				case 'percentage':
					$discountPrice = $itemPrice * $this->amount / 100;
					break;
			}

			// Don't exceed with $amount > 100%
			$discountPrice = min( $discountPrice, $itemPrice );
		}

		return $discountPrice;
	}

	public function increaseUsageCount( $save = false ) {
		$this->usageCount++;

		if ( $save ) {
			mpapp()->repositories()->coupon()->saveUsageCount( $this, $this->usageCount );
		}
	}

	/**
	 * Required for repositories. Otherwise, all/most fields will have to be
	 * made public.
	 *
	 * @example mpapp()->repositories()->coupon()->findAll(['fields' => ['id' => 'code']])
	 *
	 * @return mixed
	 */
	public function __get( string $name ) {
		$methodName = 'get' . ucfirst( $name );

		return $this->$methodName();
	}
}
