<?php

namespace MotoPress\Appointment\Payments\Gateways;

use const MotoPress\Appointment\MODE_LIVE;
use const MotoPress\Appointment\MODE_SANDBOX;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * @since 1.5.0
 */
abstract class AbstractPaymentGateway implements PaymentGatewayInterface {

	/**
	 * Unprefixed payment gateway ID.
	 *
	 * @since 1.5.0
	 * @var string
	 */
	protected $id = 'abstract';

	/**
	 * Gateway name (label) on the settings page.
	 *
	 * @since 1.5.0
	 * @var string
	 */
	protected $name = 'Abstract Payment';

	/**
	 * Public name (label) shown on the frontend.
	 *
	 * @since 1.5.0
	 * @var string
	 */
	protected $publicName = '';

	/**
	 * Public description.
	 *
	 * @since 1.5.0
	 * @var string
	 */
	protected $description = '';

	/**
	 * If true, then don't show this payment gateway on the settings page.
	 *
	 * @since 1.5.0
	 * @var bool
	 */
	protected $isInternal = false;

	/**
	 * @since 1.5.0
	 * @var bool
	 */
	protected $isEnabled = false;

	/**
	 * @since 1.5.0
	 * @var bool
	 */
	protected $hasSandboxSupport = true;

	/**
	 * @since 1.5.0
	 * @var bool
	 */
	protected $isSandbox = false;

	/**
	 * @since 1.5.0
	 */
	public function __construct() {
		$this->setupProperties();
	}

	/**
	 * @since 1.5.0
	 */
	protected function setupProperties() {

		$this->publicName  = $this->getOption( 'title', $this->publicName );
		$this->description = $this->getOption( 'description', $this->description );
		$this->isEnabled   = (bool) $this->getOption( 'enable', $this->isEnabled );
		$this->isSandbox   = (bool) $this->getOption( 'sandbox', $this->isSandbox );
	}

	/**
	 * @since 1.5.0
	 *
	 * @return array
	 */
	public function getFrontendData() {

		return array(
			'online'       => $this->isOnlinePayment(),
			'redirect_url' => array(
				'payment_received'   => mpapp()->settings()->getPaymentReceivedPageUrl(),
				'failed_transaction' => mpapp()->settings()->getFailedTransactionPageUrl(),
			),
		);
	}

	/**
	 * An empty method is provided to add wp_enqueue_script() handlers of each of the payment methods, if necessary.
	 * @see \MotoPress\Appointment\Shortcodes\AppointmentFormShortcode::enqueueScripts
	 *
	 * @return void
	 */
	public function enqueueScripts() {  }

	/**
	 * @since 1.5.0
	 *
	 * @return string
	 */
	public function getId() {
		return $this->id;
	}

	/**
	 * @since 1.5.0
	 *
	 * @return string
	 */
	public function getName() {
		return $this->name;
	}

	/**
	 * @since 1.5.0
	 *
	 * @return string
	 */
	public function getPublicName() {
		if ( ! empty( $this->publicName ) ) {
			return $this->publicName;
		} else {
			return $this->name;
		}
	}

	/**
	 * @since 1.5.0
	 *
	 * @return string
	 */
	public function getDescription() {
		return $this->description;
	}

	/**
	 * @since 1.5.0
	 *
	 * @return 'live'|'sandbox'
	 */
	public function getMode() {
		return $this->isSandbox ? MODE_SANDBOX : MODE_LIVE;
	}

	/**
	 * @since 1.5.0
	 *
	 * @return array Raw field args.
	 */
	public function getFields() {

		$backUrl = mpapp()->pages()->settings()->getUrl(
			array(
				'tab' => 'payment',
			)
		);

		$fields = array(
			$this->getOptionNameRaw( 'group' )       => array(
				'type'          => 'group',
				'label'         => $this->getName(),
				'description'   => $this->getAdminDescription(),
				'title_actions' => array(
					$backUrl => esc_html__( 'Back', 'motopress-appointment' ),
				),
			),

			$this->getOptionNameRaw( 'enable' )      => array(
				'type'    => 'checkbox',
				'label'   => esc_html__( 'Enable/Disable', 'motopress-appointment' ),
				'label2'  => esc_html__( 'Enable this payment method', 'motopress-appointment' ),
				'default' => false,
				'value'   => $this->isEnabled(),
			),

			$this->getOptionNameRaw( 'sandbox' )     => array(
				'type'        => 'checkbox',
				'label'       => esc_html__( 'Test Mode', 'motopress-appointment' ),
				'label2'      => esc_html__( 'Enable Sandbox Mode', 'motopress-appointment' ),
				'description' => esc_html__( 'Sandbox can be used to test payments.', 'motopress-appointment' ),
				'default'     => false,
				'value'       => $this->isSandbox(),
			),

			$this->getOptionNameRaw( 'title' )       => array(
				'type'         => 'text',
				'label'        => esc_html__( 'Title', 'motopress-appointment' ),
				'description'  => esc_html__( 'Payment method title that the customer will see on your website.', 'motopress-appointment' ),
				'size'         => 'regular',
				'translatable' => true,
				'value'        => $this->getPublicName(),
			),

			$this->getOptionNameRaw( 'description' ) => array(
				'type'         => 'textarea',
				'label'        => esc_html__( 'Description', 'motopress-appointment' ),
				'description'  => esc_html__( 'Payment method description that the customer will see on your website.', 'motopress-appointment' ),
				'rows'         => 3,
				'size'         => 'large',
				'translatable' => true,
				'value'        => $this->getDescription(),
			),
		);

		// Maybe remove sandbox option ("Test Mode")
		if ( ! $this->isSupportsSandbox() ) {
			$sandboxOption = $this->getOptionNameRaw( 'sandbox' );
			unset( $fields[ $sandboxOption ] );
		}

		return $fields;
	}

	/**
	 * @since 1.5.0
	 *
	 * @return string Payment gateway description on the top of the page, before
	 *      the first field.
	 */
	protected function getAdminDescription() {

		$settingsNotice = '';

		if ( ! mpapp()->settings()->isPaymentsEnabled() ) {

			$noticeMessage = wp_kses( __( '<strong>Note:</strong> Customers will see payment methods only if online payments are enabled in Appointments > Settings > Confirmation Mode.', 'motopress-appointment' ), array( 'strong' => array() ) );

			$settingsNotice = mpa_tmpl_notice( 'warning', $noticeMessage, false );
		}

		return $settingsNotice;
	}

	/**
	 * @since 1.5.0
	 *
	 * @param string $option
	 * @param mixed $default Optional. '' by default.
	 * @return mixed
	 */
	protected function getOption( $option, $default = '' ) {

		$optionName  = $this->getOptionName( $option );
		$optionValue = get_option( $optionName, $default );

		if ( $this->isTranslatableOption( $option ) ) {
			$optionValue = mpa_translate_string( $optionValue, $optionName );
		}

		return $optionValue;
	}

	/**
	 * @since 1.5.0
	 *
	 * @param string $option
	 * @return string Prefixed option name.
	 */
	protected function getOptionName( $option ) {
		return mpa_prefix( $this->getOptionNameRaw( $option ) );
	}

	/**
	 * @since 1.5.0
	 *
	 * @param string $option
	 * @return string Unprefixed option name.
	 */
	protected function getOptionNameRaw( $option ) {
		return "{$this->id}_payment_gateway_{$option}";
	}

	/**
	 * @since 1.5.0
	 *
	 * @param string $option
	 * @return bool
	 */
	protected function isTranslatableOption( $option ) {
		return in_array( $option, array( 'title', 'description' ) );
	}

	/**
	 * @since 1.5.0
	 *
	 * @return bool If true, then don't show this payment gateway on the
	 *      settings page.
	 */
	public function isInternal() {
		return $this->isInternal;
	}

	/**
	 * @since 1.5.0
	 *
	 * @return bool
	 */
	public function isEnabled() {
		return $this->isEnabled;
	}

	/**
	 * @since 1.5.0
	 *
	 * @return bool Whether gateway is enabled and supports current plugin
	 *      settings, like currency.
	 */
	public function isActive() {
		return $this->isEnabled();
	}

	/**
	 * @since 1.5.0
	 *
	 * @return bool Is sandbox enabled.
	 */
	public function isSandbox() {
		return $this->isSandbox;
	}

	/**
	 * @since 1.5.0
	 *
	 * @return bool
	 */
	public function isSupportsSandbox() {
		return $this->hasSandboxSupport;
	}

	/**
	 * @since 1.14.0
	 *
	 * @return bool
	 */
	public function isOnlinePayment() {
		return false;
	}
}
