<?php

namespace MotoPress\Appointment\Services;

use MotoPress\Appointment\Entities;
use MotoPress\Appointment\Structures\TimePeriod;
use MotoPress\Appointment\PostTypes;
use MotoPress\Appointment\Utils\BookingUtils;
use MotoPress\Appointment\Utils\ParseUtils;
use RuntimeException;
use WP_Error;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * @since 1.0
 */
class BookingService {

	/**
	 * @param array $order
	 * @return Entities\Booking|WP_Error
	 *
	 * @since 1.0
	 */
	public function createBooking( $order ) {

		if ( ! is_array( $order ) ) {
			return new WP_Error( 'invalid_input', esc_html__( 'Unable to make a reservation. Your order seems to be empty.', 'motopress-appointment' ) );
		}

		// Create reservations
		$reservations = array();

		if ( isset( $order['items'] ) ) {
			foreach ( $order['items'] as $item ) {
				$reservation = $this->createReservation( $item );

				if ( is_wp_error( $reservation ) ) {
					// Return WP_Error
					return $reservation;
				}

				$reservations[] = $reservation;
			}
		}

		// Use preset booking ID
		$bookingId = isset( $order['payment_details']['booking_id'] )
			? ParseUtils::parseId( $order['payment_details']['booking_id'] )
			: 0;

		// If booking already had a reservations and need update reservation list, then
		// delete all previously reservations because we do not want to duplicate some of them
		if ( $bookingId && isset( $order['items'] ) ) {

			$draftBooking = mpapp()->repositories()->booking()->findById( $bookingId, true );

			if ( null !== $draftBooking && ! empty( $draftBooking->getReservations() ) ) {

				foreach ( $draftBooking->getReservations() as $storedReservation ) {
					wp_delete_post( $storedReservation->getId(), true );
				}
			}
		}

		if ( $reservations ) {
			// Create booking
			$booking = new Entities\Booking(
				$bookingId,
				array(
					'status'        => mpapp()->settings()->getDefaultBookingStatus(),
					'reservations'  => $reservations,
					'customer'      => $this->parseCustomer( $order ),
					'customerNotes' => isset( $order['customer']['notes'] ) ? sanitize_text_field( $order['customer']['notes'] ) : '',
				)
			);
		} else {
			// get a ready booking
			$booking = mpapp()->repositories()->booking()->findById( $bookingId );

			if ( null === $booking ) {
				return new WP_Error( 'invalid_input', esc_html__( 'Unable to make a reservation. Your order seems to be empty.', 'motopress-appointment' ) );
			}
		}

		// Parse coupon
		$coupon = $this->parseCoupon( $order );

		if ( is_wp_error( $coupon ) ) {
			return $coupon;
		}

		// Apply coupon
		if ( ! is_null( $coupon ) && $coupon->isApplicableForBooking( $booking ) ) {
			$booking->setCouponId( $coupon->getId() );
		} else {
			$booking->setCouponId( 0 );
		}

		// Calculate the price
		$booking->setTotalPrice( BookingUtils::calcTotalPrice( $booking ) );

		return $booking;
	}

	/**
	 * @since 1.5.0
	 *
	 * @param array $order
	 * @param Entities\Booking $booking
	 * @return Entities\Payment|WP_Error
	 */
	public function createPayment( $order, $booking ) {

		if ( ! is_array( $order ) || ! isset( $order['payment_details'] ) ) {
			return new WP_Error( 'invalid_input', esc_html__( 'Payment details have not been passed.', 'motopress-appointment' ) );
		}

		// Parse params
		$paymentDetails = $order['payment_details'] + array(
			'gateway_id' => '',
			'currency'   => mpapp()->settings()->getCurrency(),
		);

		$gatewayId = sanitize_text_field( $paymentDetails['gateway_id'] );
		$currency  = sanitize_text_field( $paymentDetails['currency'] );

		$amount = $booking->getTotalPrice();
		if ( isset( $order['payment_details']['deposit'] ) && $order['payment_details']['deposit'] ) { // Is pay only deposit price?
			$amount = BookingUtils::calcDepositPrice( $booking );
		}

		// Check active gateways
		$activeGateways  = mpapp()->payments()->getActive(); // We can skip internal payments here
		$selectedGateway = mpapp()->payments()->getGateway( $gatewayId );

		if ( 'free' === $gatewayId && 0 < $amount ) {

			return new WP_Error( 'invalid_input', esc_html__( 'Unable to create a free payment on non-free order.', 'motopress-appointment' ) );

		} elseif ( ! array_key_exists( $gatewayId, $activeGateways ) && 'free' !== $gatewayId ) {

			return new WP_Error( 'invalid_input', esc_html__( 'Payment method is not valid.', 'motopress-appointment' ) );

		} elseif ( is_null( $selectedGateway ) ) {

			return new WP_Error( 'invalid_input', esc_html__( 'Payment method is not valid.', 'motopress-appointment' ) );
		}

		$payment = new Entities\Payment(
			$booking->getExpectingPaymentId(),
			array(
				'status'      => PostTypes\Statuses\PaymentStatuses::STATUS_PENDING,
				'bookingId'   => $booking->getId(),
				'amount'      => $amount,
				'currency'    => $currency,
				'gatewayId'   => $gatewayId,
				'gatewayMode' => $selectedGateway->getMode(),
			)
		);

		return $payment;
	}

	/**
	 * @param array $item Order item
	 * @return Entities\Reservation|WP_Error
	 *
	 * @since 1.0
	 */
	public function createReservation( $item ) {

		$reservation = null;

		try {
			$this->requireFields( $item );

			// Required properties
			$service = $this->parseService( $item );
			$date    = $this->parseDate( $item );
			$time    = $this->parseTime( $item );

			$time->setDate( $date );

			// Optional properties
			$employee = $this->parseEmployee( $item, $service, $date );
			$location = $this->parseLocation( $item, $employee, $date );

			// Prepare other properties
			$capacity = $this->parseCapacity( $item, $service, $employee );

			$price = $service->getPrice( $employee->getId(), $capacity );

			$bufferTime = mpa_add_buffer_time( clone $time, $service );

			$reservationFields = array(
				'price'       => $price,
				'date'        => $date,
				'serviceTime' => $time,
				'bufferTime'  => $bufferTime,
				'serviceId'   => $service->getId(),
				'employeeId'  => $employee->getId(),
				'locationId'  => $location->getId(),
				'capacity'    => $capacity,
			);

			if ( ! empty( $item['uid'] ) ) {
				// But don't add "" and replace default UUID4 key
				$reservationFields['uid'] = sanitize_text_field( $item['uid'] );
			}

			$reservation = new Entities\Reservation( 0, $reservationFields );

		} catch ( RuntimeException $error ) {
			return new WP_Error( 'invalid_input', $error->getMessage() );
		}

		return $reservation;
	}

	/**
	 * @param array $item
	 *
	 * @throws RuntimeException
	 *
	 * @since 1.0
	 */
	protected function requireFields( $item ) {
		if ( ! isset( $item['service_id'], $item['date'], $item['time'] ) ) {
			throw new RuntimeException( esc_html__( 'Unable to make a reservation. Your order misses some of the required data.', 'motopress-appointment' ) );
		}
	}

	/**
	 * @param array $item
	 * @return Entities\Service
	 *
	 * @throws RuntimeException
	 *
	 * @since 1.0
	 */
	protected function parseService( $item ) {
		$serviceId = absint( $item['service_id'] );
		$service   = mpa_get_service( $serviceId );

		if ( is_null( $service ) ) {
			// Translators: %d: ID of the post.
			throw new RuntimeException( sprintf( esc_html__( 'Unable to make a reservation. There is no such service with ID #%d.', 'motopress-appointment' ), $serviceId ) );
		}

		return $service;
	}

	/**
	 * @param array $item
	 * @return \DateTime
	 *
	 * @throws RuntimeException
	 *
	 * @since 1.0
	 */
	protected function parseDate( $item ) {
		$date = mpa_parse_date( $item['date'] );

		if ( ! $date ) {
			throw new RuntimeException( esc_html__( 'Unable to make a reservation. The date value is broken.', 'motopress-appointment' ) );
		}

		return $date;
	}

	/**
	 * @param array $item
	 * @return TimePeriod
	 *
	 * @throws RuntimeException
	 *
	 * @since 1.0
	 */
	protected function parseTime( $item ) {
		$time = TimePeriod::createFromPeriod( $item['time'] );

		if ( ! $time ) {
			throw new RuntimeException( esc_html__( 'Unable to make a reservation. The time value is broken.', 'motopress-appointment' ) );
		}

		return $time;
	}

	/**
	 * @param array $item
	 * @param Entities\Service $service
	 * @param \DateTime $date
	 * @return Entities\Employee
	 *
	 * @throws RuntimeException
	 *
	 * @since 1.0
	 */
	protected function parseEmployee( $item, $service, $date ) {

		if ( isset( $item['employee_id'] ) ) {
			// Just find the selected employee
			$employeeId = absint( $item['employee_id'] );
			$employee   = mpa_get_employee( $employeeId );

		} else {
			// Search for employee
			$locationId = isset( $item['location_id'] ) ? absint( $item['location_id'] ) : 0;
			$employee   = null;

			foreach ( $service->getEmployeeIds() as $employeeId ) {
				// Filter all employees that work at location $locationId for current $date
				if ( $locationId != 0 ) {
					$schedule = mpapp()->repositories()->schedule()->findByEmployee( $employeeId );

					if ( is_null( $schedule ) || ! $schedule->isWorkingAt( $locationId, $date ) ) {
						continue;
					}
				}

				$employee = mpa_get_employee( $employeeId );

				if ( ! is_null( $employee ) ) {
					break; // Found one
				}
			} // For each employee
		}

		if ( is_null( $employee ) ) {
			throw new RuntimeException( esc_html__( 'Unable to make a reservation. No employee found for your request.', 'motopress-appointment' ) );
		}

		return $employee;
	}

	/**
	 * @param array $item
	 * @param Entities\Employee $employee
	 * @param \DateTime $date
	 * @return Entities\Location
	 *
	 * @throws RuntimeException
	 *
	 * @since 1.0
	 */
	protected function parseLocation( $item, $employee, $date ) {

		if ( isset( $item['location_id'] ) ) {
			// Just find the selected location
			$locationId = absint( $item['location_id'] );
			$location   = mpa_get_location( $locationId );

		} else {
			// Search for location
			$schedule  = mpapp()->repositories()->schedule()->findByEmployee( $employee->getId() );
			$locations = ! is_null( $schedule ) ? $schedule->getLocationIdsForDate( $date ) : array();
			$location  = ! empty( $locations ) ? mpa_get_location( $locations[0] ) : null;
		}

		if ( is_null( $location ) ) {
			throw new RuntimeException( esc_html__( 'Unable to make a reservation. No places left for this service.', 'motopress-appointment' ) );
		}

		return $location;
	}

	/**
	 * @since 1.3.1
	 *
	 * @param array $item
	 *     @param int  $item['capacity']     Optional. 1 by default.
	 *     @param int  $item['bring_people'] Optional. The number of additional
	 *                                       guests. 0 by default.
	 * @param Entities\Service $service
	 * @param Entities\Employee $employee
	 * @return int Valid capacity for the service.
	 */
	protected function parseCapacity( $item, $service, $employee ) {

		$item += array(
			'capacity'     => $service->getMinCapacity( $employee->getId() ),
			'bring_people' => 0,
		);

		$capacity = $item['capacity'] + $item['bring_people'];

		$minCapacity = $service->getMinCapacity( $employee->getId() );
		$maxCapacity = $service->getMaxCapacity( $employee->getId() );

		$capacity = mpa_limit( $capacity, $minCapacity, $maxCapacity );

		return $capacity;
	}

	/**
	 * @param array $order
	 * @return Entities\Customer
	 *
	 * @since 1.0
	 */
	protected function parseCustomer( $order ) {

		$customerInfo = array();

		if ( isset( $order['customer'] ) && is_array( $order['customer'] ) ) {
			$customerInfo = array_map( 'sanitize_text_field', $order['customer'] );
		}

		return new Entities\Customer( 0, $customerInfo );
	}

	/**
	 * @param array $order
	 * @return Entities\Coupon|null|WP_Error
	 *
	 * @since 1.11.0
	 */
	protected function parseCoupon( $order ) {

		if ( ! isset( $order['payment_details']['coupon_code'] ) || ! $order['payment_details']['coupon_code'] ) {
			return null;
		}

		$couponCode = sanitize_text_field( $order['payment_details']['coupon_code'] );

		if ( $couponCode ) {
			$coupon = mpapp()->repositories()->coupon()->findByCode( $couponCode );
		} else {
			return new WP_Error( 'invalid_input', esc_html__( 'Invalid parameter: coupon ID or code are not set.', 'motopress-appointment' ) );
		}

		if ( is_null( $coupon ) ) {
			return new WP_Error( 'invalid_input', esc_html__( 'Coupon not found.', 'motopress-appointment' ) );
		}

		if ( $coupon->isExpired() ) {
			return new WP_Error( 'invalid_input', esc_html__( 'This coupon has expired.', 'motopress-appointment' ) );
		}

		if ( $coupon->isExceededUsageLimit() ) {
			return new WP_Error( 'invalid_input', esc_html__( 'Coupon usage limit has been reached.', 'motopress-appointment' ) );
		}

		return $coupon;
	}

	/**
	 * @param Entities\Booking $booking
	 * @return int|false New booking ID or false.
	 *
	 * @since 1.0
	 */
	public function saveBooking( $booking ) {

		if ( ! $booking->getId() ) {
			$postId = wp_insert_post(
				array(
					'post_type'   => PostTypes\BookingPostType::POST_TYPE,
					'post_status' => $booking->getStatus(),
				),
				true
			);

			if ( is_wp_error( $postId ) ) {
				return false;
			} else {
				$booking->setId( $postId );
			}
		} else {
			$postId = $booking->getId();

			mpa_update_post_status( $postId, $booking->getStatus() );
		}

		// Update booking meta fields
		mpa_add_post_uid( $postId, $booking->getUid() );

		update_post_meta( $postId, '_mpa_total_price', $booking->getTotalPrice() );
		update_post_meta( $postId, '_mpa_coupon_id', $booking->getCouponId() );

		// Update customer info
		$booking->getCustomer()->setId( $postId );

		update_post_meta( $postId, '_mpa_customer_name', $booking->getCustomer()->getName() );
		update_post_meta( $postId, '_mpa_customer_email', $booking->getCustomer()->getEmail() );
		update_post_meta( $postId, '_mpa_customer_phone', $booking->getCustomer()->getPhone() );
		update_post_meta( $postId, '_mpa_customer_notes', $booking->getCustomerNotes() );

		// Save reservations
		foreach ( $booking->getReservations() as $reservation ) {
			$reservation->setBookingId( $postId );

			$this->saveReservation( $reservation );
		}

		// Notify others about new booking
		if ( false !== $postId && 'auto-draft' != $booking->getStatus() ) {
			/**
			 * Fires after actions "mpa_new_booking_created" and "mpa_booking_{$newStatus}".
			 *
			 * @see \MotoPress\Appointment\PostTypes\Statuses\BookingStatuses::notifyTransition()
			 *
			 * @since 1.1.0
			 */
			do_action( 'mpa_booking_placed_by_user', $booking );
		}

		return $postId;
	}

	/**
	 * @param Entities\Reservation $reservation
	 * @return int|false Post ID or false.
	 */
	public function saveReservation( Entities\Reservation $reservation ) {

		// Create new reservation in wp_posts, if it does not exist
		if ( ! $reservation->getId() ) {
			$postId = wp_insert_post(
				array(
					'post_type'   => PostTypes\ReservationPostType::POST_TYPE,
					'post_status' => 'publish',
					'post_parent' => $reservation->getBookingId(),
				)
			);

			if ( is_wp_error( $postId ) || 0 === $postId ) {
				return false;
			} else {
				$reservation->setId( $postId );
			}
		} else {
			$postId = $reservation->getId();
		}

		update_post_meta( $postId, '_mpa_uid', $reservation->getUid() );
		update_post_meta( $postId, '_mpa_price', $reservation->getPrice() );
		update_post_meta( $postId, '_mpa_date', mpa_format_date( $reservation->getDate(), 'internal' ) );
		update_post_meta( $postId, '_mpa_service_time', $reservation->getServiceTime()->toString( 'internal' ) );
		update_post_meta( $postId, '_mpa_buffer_time', $reservation->getBufferTime()->toString( 'internal' ) );
		update_post_meta( $postId, '_mpa_service', $reservation->getServiceId() );
		update_post_meta( $postId, '_mpa_employee', $reservation->getEmployeeId() );
		update_post_meta( $postId, '_mpa_location', $reservation->getLocationId() );
		update_post_meta( $postId, '_mpa_capacity', $reservation->getCapacity() );

		return $postId;
	}

	/**
	 * @see \MotoPress\Appointment\Metaboxes\Booking\BookingPriceMetabox
	 */
	public function updateReservations( Entities\Booking $booking, array $newReservations ) {

		$newUids = array();

		foreach ( $newReservations as $newReservation ) {

			$newUids[] = $newReservation->getUid();
		}

		// Remove existing reservations
		foreach ( $booking->getReservations() as $reservation ) {

			// Reservation was removed from booking?
			if ( ! in_array( $reservation->getUid(), $newUids ) ) {
				wp_delete_post( $reservation->getId(), true );
			}
		}

		// Update / add new reservations
		foreach ( $newReservations as $reservation ) {
			// Try to find ID by UID
			if ( ! $reservation->getId() ) {
				$reservation->setId( mpapp()->repositories()->reservation()->findIdByMeta( 'uid', $reservation->getUid() ) );
			}

			$this->saveReservation( $reservation );
		}

		$booking->setReservations( $newReservations );

		/**
		 * @since 1.10.0
		 * @param Entities\Booking $booking
		 */
		do_action( 'mpa_reservations_updated', $booking );
	}

	/**
	 * @since 1.5.0
	 *
	 * @param Entities\Payment $payment
	 * @return int|false New payment ID or false.
	 */
	public function savePayment( $payment ) {

		if ( ! $payment->getId() ) {
			$postId = wp_insert_post(
				array(
					'post_type'   => PostTypes\PaymentPostType::POST_TYPE,
					'post_status' => $payment->getStatus(),
					'post_parent' => $payment->getBookingId(),
				),
				true
			);

			if ( is_wp_error( $postId ) ) {
				return false;
			} else {
				$payment->setId( $postId );
			}
		} else {
			$postId = $payment->getId();

			mpa_update_post_status( $postId, $payment->getStatus() );
		}

		mpa_add_post_uid( $postId, $payment->getUid() );

		update_post_meta( $postId, '_mpa_amount', $payment->getAmount() );
		update_post_meta( $postId, '_mpa_currency', $payment->getCurrency() );
		update_post_meta( $postId, '_mpa_gateway_id', $payment->getGatewayId() );
		update_post_meta( $postId, '_mpa_gateway_mode', $payment->getGatewayMode() );
		update_post_meta( $postId, '_mpa_payment_method', $payment->getPaymentMethod() );
		update_post_meta( $postId, '_mpa_transaction_id', $payment->getTransactionId() );

		return $postId;
	}
}
