<?php

declare(strict_types=1);

namespace MotoPress\Appointment\Utils;

use MotoPress\Appointment\Entities\Booking;
use MotoPress\Appointment\Entities\Reservation;
use MotoPress\Appointment\Services\TimeSlotService;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class BookingUtils {

	/**
	 * Calculates new price for actual data (reservations, coupons etc.).
	 *
	 * @param Booking $booking
	 * @return float
	 */
	public static function calcTotalPrice( Booking $booking ): float {
		$totalPrice = 0.0;

		foreach ( $booking->getReservations() as $reservation ) {
			$totalPrice += $reservation->getPrice();
		}

		// Calc discount price
		if ( $booking->hasCoupon() ) {
			$coupon = $booking->getCoupon();

			if ( ! is_null( $coupon ) ) {
				$discountPrice = $coupon->calcDiscountAmount( $booking );
				$totalPrice    = max( 0, $totalPrice - $discountPrice );
			}
		}

		return $totalPrice;
	}

	/**
	 * Calculates and saves the new price (including coupon discount).
	 */
	public static function recalculateTotalPrice( Booking $booking ) {
		$newPrice = static::calcTotalPrice( $booking );
		$newPrice = max( $newPrice, $booking->getPaidPrice() ); // Can't set less than already paid

		if ( $newPrice !== $booking->getTotalPrice() ) {
			mpapp()->repositories()->booking()->savePrice( $booking, $newPrice );
		}
	}

	/**
	 * Calculates deposit price for booked services.
	 *
	 * @param Booking $booking
	 *
	 * @return float
	 * @todo better move the calculation of coupons into an independent function.
	 * It is possible to move the coupon calculation from the static::calcTotalPrice() method into a separate method and use it in the current method.
	 *
	 */
	public static function calcDepositPrice( Booking $booking ): float {
		$depositPrice = 0.0;

		foreach ( $booking->getReservations() as $reservation ) {

			$serviceId = $reservation->getServiceId();
			$service   = mpapp()->repositories()->service()->findById( $serviceId );

			$reservationPrice           = $reservation->getPrice();
			$priceAfterPossibleDiscount = $reservationPrice;
			if ( $booking->hasCoupon() ) {
				$coupon = $booking->getCoupon();

				if ( ! is_null( $coupon ) && $coupon->isValid() ) {
					$priceAfterPossibleDiscount = max( 0, $reservation->getPrice() - $coupon->calcDiscountForReservation( $reservation ) );
				}
			}

			$deposit = $priceAfterPossibleDiscount;
			switch ( $service->getDepositType() ) {
				case 'disabled':
					$deposit = $priceAfterPossibleDiscount;
					break;
				case 'fixed':
					$deposit = $service->getDepositAmount();
					break;
				case 'percentage':
					$deposit = $priceAfterPossibleDiscount * $service->getDepositAmount() / 100;
					break;
			}

			if ( $deposit > $priceAfterPossibleDiscount ) {
				$deposit = $priceAfterPossibleDiscount;
			}

			$depositPrice += $deposit;
		}

		return $depositPrice;
	}

	/**
	 * @since 1.15.2
	 * @protected
	 *
	 * @param array $args get_posts() search parameters
	 *
	 * @return array
	 */
	public static function excludeBookingReservationsHandler( $args ) {

		global $excludeBooking;

		if ( isset( $args['post_parent__not_in'] ) &&
			! is_array( $args ) ) {
			return $args;
		}

		$args['post_parent__not_in'][] = $excludeBooking->getId();

		remove_filter( '', array( __CLASS__, 'excludeBookingReservationsHandler' ) );

		return $args;
	}

	/**
	 * @since 1.15.2
	 *
	 * @return void
	 */
	protected static function excludeCurrentBookingReservations( Booking $booking ) {
		global $excludeBooking;
		$excludeBooking = $booking;

		$postType = mpapp()->postTypes()->reservation()->getPostType();
		add_filter(
			"{$postType}_repository_get_posts_query_args",
			array(
				__CLASS__,
				'excludeBookingReservationsHandler',
			)
		);
	}

	/**
	 * @since 1.15.2
	 *
	 * @return bool
	 */
	protected static function isStillAvailableTimeSlot( Reservation $reservation ) {
		$service = mpa_get_service( $reservation->getServiceId() );
		$date    = $reservation->getDate();

		$args = array(
			'employees' => array( $reservation->getEmployeeId() ),
			'locations' => array( $reservation->getLocationId() ),
		);

		$timeSlotService = new TimeSlotService( $service, $args );

		// Block reserved periods
		$timeSlotService->blockPeriod( $date, $date );

		return $timeSlotService->isAvailableTimeSlot( $reservation->getServiceTime() );
	}

	/**
	 * @since 1.15.2
	 *
	 * @return bool
	 */
	public static function isStillAvailableTimeSlots( Booking $booking ) {
		self::excludeCurrentBookingReservations( $booking );

		$reservations = $booking->getReservations();

		foreach ( $reservations as $reservation ) {
			if ( ! self::isStillAvailableTimeSlot( $reservation ) ) {
				return false;
			}
		}

		return true;
	}
}
